' Make sure we're running on Vectrex32 version 1.10 or newer.
' We're using 3D graphics features that don't exist on older
' versions.
if Version() < 110 then
    print "Upgrade the Vectrex32 firmware to version 1.10 or newer."
    stop
endif

' Set the frame rate to 120 frames per second
call SetFrameRate(120)

' Display instructions for using the program
textSize = {40, 5}
call TextSizeSprite(textSize)
instructions = {{-50, 90, "INSTRUCTIONS"}, _
                {-80, 70, "BTN 1 CREATES PYRAMID"}, _
                {-80, 50, "JOY PITCHES AND YAWS"}, _
                {-80, 30, "JOY + BTN 2 ROLLS"}, _
                {-80, 10, "JOY + BTN 3 MOVES CAMERA"}, _
                {-80, -10, "BTN 4 EXITS"}, _
                {-80, -30, "PRESS BTN 1 TO START"}}
        
call TextListSprite(instructions)

' Wait for none of the buttons to be pressed (in case this
' program was launched by pressing a button on a menu: we 
' should wait until the user has had time to release the button)
controls = WaitForFrame(JoystickNone, Controller1, JoystickNone)
while controls[1, 3] or controls[1, 4] or controls[1, 5] or controls[1, 6] 
    controls = WaitForFrame(JoystickNone, Controller1, JoystickNone)
endwhile

' Wait for button 1 or button 4 to be pressed
controls = WaitForFrame(JoystickNone, Controller1, JoystickNone)
while controls[1, 3] = 0 and controls[1, 6] = 0
    controls = WaitForFrame(JoystickNone, Controller1, JoystickNone)
endwhile

' If button 4 was pressed, exit
if controls[1, 6] then
    stop
endif

' We don't need the instructions anymore, so clear them
call ClearScreen()

' The camera is placed at (0, 0, 0) by default. But we're going to move it
' around so we'll use our own translation array to specify its position
cameraTrans = {0, 0, 0}
call CameraTranslate(cameraTrans)

' When the user creates a new pyramid, we'll place it at a random location
' and orientation
call Randomize

' Set the scale to 64. At a scale of 64, 324 Vectrex units will draw a line
' on the Vectrex screen that is 0.097 meters long (and we're using meters for
' our World coordinates)
call ScaleSprite(64, 324 / 0.097)

' Define the vectors for the pyramid. Note that each row of the array has
' X, Y, and Z coordinates
pyramidWidth = 2
pyramidHeight = 2
pyramidLines = { _
    {MoveTo, 0, pyramidHeight, 0}, _
    {DrawTo, pyramidWidth / 2, 0, 0}, _
    {DrawTo, 0, 0, pyramidWidth / 2}, _
    {DrawTo, -pyramidWidth / 2, 0, 0}, _
    {DrawTo, 0, 0, -pyramidWidth / 2}, _
    {DrawTo, pyramidWidth / 2, 0, 0}, _
    {MoveTo, 0, pyramidHeight, 0}, _
    {DrawTo, -pyramidWidth / 2, 0, 0}, _ 
    {MoveTo, 0, pyramidHeight, 0}, _
    {DrawTo, 0, 0, -pyramidWidth / 2}, _
    {MoveTo, 0, pyramidHeight, 0}, _
    {DrawTo, 0, 0, pyramidWidth / 2}}

' Create the first pyramid
pyramids = NewPyramid()

' The user pressed button 1 to exit the menu. Once we're in the loop, below,
' button 1 will create new pyramids. But we don't want to read a single button
' press twice. So we ignore button 1 for 1/2 second
ignoreButton1 = GetFrameRate() / 2

' Loop forever
while true
    ' Draw the fram and get the controller actions
    controls = WaitForFrame(JoystickDigital, Controller1, JoystickX + JoystickY)
    
    ' If button 4 was pressed, exit
    if controls[1, 6] then
        stop
    endif
    
    ' If button 1 was pressed, create a new pyramid
    if controls[1, 3] and ignoreButton1 = 0 then
        ' Before each 3D sprite, we should return the pen to the origin.
        ' Otherwise, Vectrex32 wouldn't be able to draw it in the right position
        ' on the screen.
        call ReturnToOriginSprite()
        ' Create a new pyramid and add it to the array of pyramid sprite handles.
        ' (Note that the NewPyramid function returns an array with one element in it,
        ' the handle for the new pyramid)
        pyramids = AppendArrays(pyramids, NewPyramid())
        ' Ignore button 1 for 1/2 second
        ignoreButton1 = GetFrameRate() / 2
    endif
    
    ' If we're ignoring button 1
    if ignoreButton1 then
        ' Count down the number of frames to ignore it
        ignoreButton1 = ignoreButton1 - 1
    endif
    
    ' If button 3 is not pressed
    if controls[1, 5] = 0 then
        ' If button 2 is not pressed
        if controls[1, 4] = 0 then
            ' Go through all the pyramids
            for i = 1 to UBound(pyramids)
                ' Yaw the pyramid based on the joystick's X position
                if controls[1, 1] < 0 then
                    call SpriteRotate(pyramids[i], 0, 0, -1)
                elseif controls[1, 1] > 0
                    call SpriteRotate(pyramids[i], 0, 0, 1)
                endif
                ' Pitch the pyramid based on Y
                if controls[1, 2] < 0 then
                    call SpriteRotate(pyramids[i], -1, 0, 0)
                elseif controls[1, 2] > 0
                    call SpriteRotate(pyramids[i], 1, 0, 0)
                endif
            next i
        
        ' Else (button 2 is pressed)
        else
            ' Go through all the pyramids
            for i = 1 to UBound(pyramids)
                ' Roll the pyramid based on the joystick
                if controls[1, 1] < 0 then
                    call SpriteRotate(pyramids[i], 0, -1, 0)
                elseif controls[1, 1] > 0
                    call SpriteRotate(pyramids[i], 0, 1, 0)
                endif
            next i
        endif
    
    ' Else (button 3 is pressed)
    else
        ' If button 2 is not pressed
        if controls[1, 4] = 0 then
            ' Change the camera's yaw
            if controls[1, 1] < 0 then
                call CameraRotate(0, 0, -1)
            elseif controls[1, 1] > 0
                call CameraRotate(0, 0, 1)
            endif
            ' Change the camera's pitch
            if controls[1, 2] < 0 then
                call CameraRotate(-1, 0, 0)
            elseif controls[1, 2] > 0
                call CameraRotate(1, 0, 0)
            endif
        ' Else (button 2 is pressed)
        else
            ' Change the camera's roll
            if controls[1, 1] < 0 then
                call CameraRotate(0, -1, 0)
            elseif controls[1, 1] > 0
                call CameraRotate(0, 1, 0)
            endif
            camRotation = CameraGetRotation()
            ' Change the camera's position
            v = {0, 0, 0}
            if controls[1, 2] < 0 then
                v = {0, 0, -1}
            elseif controls[1, 2] > 0
                v = {0, 0, 1}
            endif
            v = VectorRotate(v, camRotation[1], camRotation[2], camRotation[3])
            cameraTrans[1] = cameraTrans[1] + v[1]
            cameraTrans[2] = cameraTrans[2] + v[2]
            cameraTrans[3] = cameraTrans[3] + v[3]
        endif
    endif
endwhile

function NewPyramid()
    dim pyramid[1]
    ' Create a sprite from the pyramidLines. pyramidLines is a global variable and
    ' it would be more elegant to define it as a local variable in this function
    ' instead. But that would cause it to be created every time we enter this function
    ' and destroy it when we exit this function. So having it be a global variable is
    ' more efficient.
    pyramid[1] = Lines3DSprite(pyramidLines)
    ' Position the pyramid randomly in the world. First, get Z, X, and Y coordinates
    ' Choose a distance from us between 50 and 150 meters
    z = rand() mod 100 + 50
    ' Choose an apparent position on the screen in meters (the screen is
    ' about 0.15 meters by 0.18 meters)
    x = (rand() mod (150 - pyramidWidth) - 75) / 1000.0
    y = (rand() mod (180 - pyramidHeight) - 90) / 1000.0
    ' Scale the apparent position on the screen to an actual translation based on 
    ' the pyramid's z distance (and assuming the user sits 0.5 meters from the screen)
    x = x / 0.5 * (z + 0.5)
    y = y / 0.5 * (z + 0.5)
    ' We've just chosen coordinates that would be in front of the camera if the camera
    ' were at (0, 0, 0) with no rotation. But the user might've moved or rotated the
    ' camera. So use the X, Y, and Z coordinates to make a vector and rotate that
    ' vector the same way the camera is rotated.
    cameraAngles = CameraGetRotation()
    vector = VectorRotate({x, y, z}, cameraAngles[1], cameraAngles[2], cameraAngles[3])
    ' Now translate the pyramid's coordinates to match the camera's position
    vector[1] = vector[1] + cameraTrans[1]
    vector[2] = vector[2] + cameraTrans[2]
    vector[3] = vector[3] + cameraTrans[3]
    call SpriteTranslate(pyramid[1], vector)
    ' Set the rotation of the pyramid to random values
    call SpriteRotate(pyramid[1], rand() mod 360, rand() mod 360, rand() mod 360)
    ' Return the array of one pyramid handle
    return pyramid
endfunction
